@@ -15,6 +15,7 @@
 #include <string.h>
 
 #define DEFAULT_JPEG_QUALITY 90
+#define DECODE_ALL_FRAMES -1
 
 #define NEXTARG()                                                     \
     if (((argIndex + 1) == argc) || (argv[argIndex + 1][0] == '-')) { \
@@ -30,47 +31,79 @@ static void syntax(void)
     printf("Options:\n");
     printf("    -h,--help         : Show syntax help\n");
     printf("    -V,--version      : Show the version number\n");
-    printf("    -j,--jobs J       : Number of jobs (worker threads, default: 1. Use \"all\" to use all available cores)\n");
-    printf("    -c,--codec C      : AV1 codec to use (choose from versions list below)\n");
-    printf("    -d,--depth D      : Output depth [8,16]. (PNG only; For y4m, depth is retained, and JPEG is always 8bpc)\n");
-    printf("    -q,--quality Q    : Output quality [0-100]. (JPEG only, default: %d)\n", DEFAULT_JPEG_QUALITY);
-    printf("    --png-compress L  : Set PNG compression level (PNG only; 0-9, 0=none, 9=max). Defaults to libpng's builtin default.\n");
-    printf("    -u,--upsampling U : Chroma upsampling (for 420/422). automatic (default), fastest, best, nearest, or bilinear\n");
+    printf("    -j,--jobs J       : Number of jobs (worker threads), or 'all' to potentially use as many cores as possible. (Default: all)\n");
+    printf("    -c,--codec C      : Codec to use (choose from versions list below)\n");
+    printf("    -d,--depth D      : Output depth, either 8 or 16. (PNG only; For y4m, depth is retained, and JPEG is always 8bpc)\n");
+    printf("    -q,--quality Q    : Output quality in 0..100. (JPEG only, default: %d)\n", DEFAULT_JPEG_QUALITY);
+    printf("    --png-compress L  : PNG compression level in 0..9 (PNG only; 0=none, 9=max). Defaults to libpng's builtin default\n");
+    printf("    -u,--upsampling U : Chroma upsampling (for 420/422). One of 'automatic' (default), 'fastest', 'best', 'nearest', or 'bilinear'\n");
     printf("    -r,--raw-color    : Output raw RGB values instead of multiplying by alpha when saving to opaque formats\n");
     printf("                        (JPEG only; not applicable to y4m)\n");
-    printf("    --index I         : When decoding an image sequence or progressive image, specify which frame index to decode (Default: 0)\n");
+    printf("    --index I         : When decoding an image sequence or progressive image, specify which frame index to decode, where the first frame has index 0, or 'all' to decode all frames. (Default: 0)\n");
     printf("    --progressive     : Enable progressive AVIF processing. If a progressive image is encountered and --progressive is passed,\n");
     printf("                        avifdec will use --index to choose which layer to decode (in progressive order).\n");
     printf("    --no-strict       : Disable strict decoding, which disables strict validation checks and errors\n");
     printf("    -i,--info         : Decode all frames and display all image information instead of saving to disk\n");
+    printf("    --icc FILENAME    : Provide an ICC profile payload (implies --ignore-icc)\n");
     printf("    --ignore-icc      : If the input file contains an embedded ICC profile, ignore it (no-op if absent)\n");
-    printf("    --size-limit C    : Specifies the image size limit (in total pixels) that should be tolerated.\n");
-    printf("                        Default: %u, set to a smaller value to further restrict.\n", AVIF_DEFAULT_IMAGE_SIZE_LIMIT);
-    printf("  --dimension-limit C : Specifies the image dimension limit (width or height) that should be tolerated.\n");
-    printf("                        Default: %u, set to 0 to ignore.\n", AVIF_DEFAULT_IMAGE_DIMENSION_LIMIT);
-    printf("    --                : Signals the end of options. Everything after this is interpreted as file names.\n");
+    printf("    --size-limit C    : Maximum image size (in total pixels) that should be tolerated. (Default: %u)\n",
+           AVIF_DEFAULT_IMAGE_SIZE_LIMIT);
+    printf("  --dimension-limit C : Maximum image dimension (width or height) that should be tolerated.\n");
+    printf("                        Set to 0 to ignore. (Default: %u)\n", AVIF_DEFAULT_IMAGE_DIMENSION_LIMIT);
+    printf("    --                : Signal the end of options. Everything after this is interpreted as file names.\n");
     printf("\n");
     avifPrintVersions();
 }
 
+avifBool avifWriteToFile(avifAppFileFormat outputFormat,
+                         const char * outputFilename,
+                         avifImage * image,
+                         avifBool rawColor,
+                         int jpegQuality,
+                         int pngCompressionLevel,
+                         int requestedDepth,
+                         int chromaUpsampling)
+{
+    if (outputFormat == AVIF_APP_FILE_FORMAT_Y4M) {
+        if (image->icc.size || image->exif.size || image->xmp.size) {
+            fprintf(stderr, "Warning: metadata dropped when saving to y4m.\n");
+        }
+        return y4mWrite(outputFilename, image);
+    } else if (outputFormat == AVIF_APP_FILE_FORMAT_JPEG) {
+        // Bypass alpha multiply step during conversion
+        if (rawColor) {
+            image->alphaPremultiplied = AVIF_TRUE;
+        }
+        return avifJPEGWrite(outputFilename, image, jpegQuality, chromaUpsampling);
+    } else if (outputFormat == AVIF_APP_FILE_FORMAT_PNG) {
+        return avifPNGWrite(outputFilename, image, requestedDepth, chromaUpsampling, pngCompressionLevel);
+    } else {
+        fprintf(stderr, "Unsupported output file extension: %s\n", outputFilename);
+        return AVIF_FALSE;
+    }
+}
+
 int main(int argc, char * argv[])
 {
     const char * inputFilename = NULL;
     const char * outputFilename = NULL;
     int requestedDepth = 0;
-    int jobs = 1;
+    int jobs = -1;
     int jpegQuality = DEFAULT_JPEG_QUALITY;
     int pngCompressionLevel = -1; // -1 is a sentinel to avifPNGWrite() to skip calling png_set_compression_level()
     avifCodecChoice codecChoice = AVIF_CODEC_CHOICE_AUTO;
     avifBool infoOnly = AVIF_FALSE;
     avifChromaUpsampling chromaUpsampling = AVIF_CHROMA_UPSAMPLING_AUTOMATIC;
+    const char * iccOverrideFilename = NULL;
     avifBool ignoreICC = AVIF_FALSE;
     avifBool rawColor = AVIF_FALSE;
     avifBool allowProgressive = AVIF_FALSE;
     avifStrictFlags strictFlags = AVIF_STRICT_ENABLED;
-    uint32_t frameIndex = 0;
+    int frameIndex = 0;                        // Decode the first frame by default.
+    avifBool frameIndexSpecified = AVIF_FALSE; // Whether the --index flag was passed.
     uint32_t imageSizeLimit = AVIF_DEFAULT_IMAGE_SIZE_LIMIT;
     uint32_t imageDimensionLimit = AVIF_DEFAULT_IMAGE_DIMENSION_LIMIT;
+    avifRWData iccOverride = AVIF_DATA_EMPTY;
 
     if (argc < 2) {
         syntax();
@@ -124,7 +157,7 @@ int main(int argc, char * argv[])
             } else {
                 const char * codecName = avifCodecName(codecChoice, AVIF_CODEC_FLAG_CAN_DECODE);
                 if (codecName == NULL) {
-                    fprintf(stderr, "ERROR: AV1 Codec cannot decode: %s\n", arg);
+                    fprintf(stderr, "ERROR: Codec cannot decode: %s\n", arg);
                     return 1;
                 }
             }
@@ -173,11 +206,20 @@ int main(int argc, char * argv[])
             allowProgressive = AVIF_TRUE;
         } else if (!strcmp(arg, "--index")) {
             NEXTARG();
-            frameIndex = (uint32_t)atoi(arg);
+            if (!strcmp(arg, "all")) {
+                frameIndex = DECODE_ALL_FRAMES;
+            } else {
+                frameIndex = (uint32_t)atoi(arg);
+            }
+            frameIndexSpecified = AVIF_TRUE;
         } else if (!strcmp(arg, "--no-strict")) {
             strictFlags = AVIF_STRICT_DISABLED;
         } else if (!strcmp(arg, "-i") || !strcmp(arg, "--info")) {
             infoOnly = AVIF_TRUE;
+        } else if (!strcmp(arg, "--icc")) {
+            NEXTARG();
+            iccOverrideFilename = arg;
+            ignoreICC = AVIF_TRUE;
         } else if (!strcmp(arg, "--ignore-icc")) {
             ignoreICC = AVIF_TRUE;
         } else if (!strcmp(arg, "--size-limit")) {
@@ -216,156 +258,220 @@ int main(int argc, char * argv[])
         ++argIndex;
     }
 
+    if (jobs == -1) {
+        jobs = avifQueryCPUCount();
+    }
+
     if (!inputFilename) {
         syntax();
         return 1;
     }
 
+    if (!inputFilename) {
+        fprintf(stderr, "Missing input filename\n");
+        syntax();
+        return 1;
+    }
+
+    avifAppFileFormat outputFormat = AVIF_APP_FILE_FORMAT_UNKNOWN;
     if (infoOnly) {
-        if (!inputFilename || outputFilename) {
+        if (outputFilename) {
+            fprintf(stderr, "ERROR: info requested (-i or --info) but output filename also provided (%s)\n", outputFilename);
             syntax();
             return 1;
         }
-
-        avifDecoder * decoder = avifDecoderCreate();
-        decoder->maxThreads = jobs;
-        decoder->codecChoice = codecChoice;
-        decoder->imageSizeLimit = imageSizeLimit;
-        decoder->imageDimensionLimit = imageDimensionLimit;
-        decoder->strictFlags = strictFlags;
-        decoder->allowProgressive = allowProgressive;
-        avifResult result = avifDecoderSetIOFile(decoder, inputFilename);
-        if (result != AVIF_RESULT_OK) {
-            fprintf(stderr, "Cannot open file for read: %s\n", inputFilename);
-            avifDecoderDestroy(decoder);
-            return 1;
-        }
-        result = avifDecoderParse(decoder);
-        if (result == AVIF_RESULT_OK) {
-            printf("Image decoded: %s\n", inputFilename);
-            avifContainerDump(decoder);
-
-            printf(" * %" PRIu64 " timescales per second, %2.2f seconds (%" PRIu64 " timescales), %d frame%s\n",
-                   decoder->timescale,
-                   decoder->duration,
-                   decoder->durationInTimescales,
-                   decoder->imageCount,
-                   (decoder->imageCount == 1) ? "" : "s");
-            if (decoder->imageCount > 1) {
-                printf(" * %s Frames: (%u expected frames)\n",
-                       (decoder->progressiveState != AVIF_PROGRESSIVE_STATE_UNAVAILABLE) ? "Progressive Image" : "Image Sequence",
-                       decoder->imageCount);
-            } else {
-                printf(" * Frame:\n");
-            }
-
-            int currIndex = 0;
-            while ((result = avifDecoderNextImage(decoder)) == AVIF_RESULT_OK) {
-                printf("   * Decoded frame [%d] [pts %2.2f (%" PRIu64 " timescales)] [duration %2.2f (%" PRIu64 " timescales)] [%ux%u]\n",
-                       currIndex,
-                       decoder->imageTiming.pts,
-                       decoder->imageTiming.ptsInTimescales,
-                       decoder->imageTiming.duration,
-                       decoder->imageTiming.durationInTimescales,
-                       decoder->image->width,
-                       decoder->image->height);
-                ++currIndex;
-            }
-            if (result == AVIF_RESULT_NO_IMAGES_REMAINING) {
-                result = AVIF_RESULT_OK;
-            } else {
-                fprintf(stderr, "ERROR: Failed to decode frame: %s\n", avifResultToString(result));
-                avifDumpDiagnostics(&decoder->diag);
-            }
-        } else {
-            fprintf(stderr, "ERROR: Failed to parse image: %s\n", avifResultToString(result));
-            avifDumpDiagnostics(&decoder->diag);
-        }
-
-        avifDecoderDestroy(decoder);
-        return result != AVIF_RESULT_OK;
     } else {
-        if (!inputFilename || !outputFilename) {
+        if (!outputFilename) {
+            fprintf(stderr, "Missing output filename\n");
             syntax();
             return 1;
         }
+        outputFormat = avifGuessFileFormat(outputFilename);
+        if (outputFormat == AVIF_APP_FILE_FORMAT_UNKNOWN) {
+            fprintf(stderr, "Cannot determine output file extension: %s\n", outputFilename);
+            return 1;
+        }
     }
 
-    printf("Decoding with AV1 codec '%s' (%d worker thread%s), please wait...\n",
+    printf("Decoding with codec '%s' (%d worker thread%s), please wait...\n",
            avifCodecName(codecChoice, AVIF_CODEC_FLAG_CAN_DECODE),
            jobs,
            (jobs == 1) ? "" : "s");
 
-    int returnCode = 0;
+    // ------ After this point, use 'goto cleanup;' in case of failure ------
+    int returnCode = 1;
     avifDecoder * decoder = avifDecoderCreate();
+    if (!decoder) {
+        fprintf(stderr, "Memory allocation failure\n");
+        goto cleanup;
+    }
     decoder->maxThreads = jobs;
     decoder->codecChoice = codecChoice;
     decoder->imageSizeLimit = imageSizeLimit;
     decoder->imageDimensionLimit = imageDimensionLimit;
     decoder->strictFlags = strictFlags;
     decoder->allowProgressive = allowProgressive;
+    if (infoOnly) {
+        decoder->imageContentToDecode = AVIF_IMAGE_CONTENT_ALL;
+    }
 
     avifResult result = avifDecoderSetIOFile(decoder, inputFilename);
     if (result != AVIF_RESULT_OK) {
         fprintf(stderr, "Cannot open file for read: %s\n", inputFilename);
-        returnCode = 1;
         goto cleanup;
     }
 
     result = avifDecoderParse(decoder);
     if (result != AVIF_RESULT_OK) {
         fprintf(stderr, "ERROR: Failed to parse image: %s\n", avifResultToString(result));
-        returnCode = 1;
-        goto cleanup;
-    }
-
-    result = avifDecoderNthImage(decoder, frameIndex);
-    if (result != AVIF_RESULT_OK) {
-        fprintf(stderr, "ERROR: Failed to decode image: %s\n", avifResultToString(result));
-        returnCode = 1;
         goto cleanup;
     }
 
     printf("Image decoded: %s\n", inputFilename);
-    printf("Image details:\n");
-    avifImageDump(decoder->image, 0, 0, decoder->progressiveState);
+    avifContainerDump(decoder);
 
-    if (ignoreICC && (decoder->image->icc.size > 0)) {
-        printf("[--ignore-icc] Discarding ICC profile.\n");
-        // This cannot fail.
-        result = avifImageSetProfileICC(decoder->image, NULL, 0);
-        assert(result == AVIF_RESULT_OK);
-    }
-
-    avifAppFileFormat outputFormat = avifGuessFileFormat(outputFilename);
-    if (outputFormat == AVIF_APP_FILE_FORMAT_UNKNOWN) {
-        fprintf(stderr, "Cannot determine output file extension: %s\n", outputFilename);
-        returnCode = 1;
-    } else if (outputFormat == AVIF_APP_FILE_FORMAT_Y4M) {
-        if (!y4mWrite(outputFilename, decoder->image)) {
-            returnCode = 1;
-        }
-    } else if (outputFormat == AVIF_APP_FILE_FORMAT_JPEG) {
-        // Bypass alpha multiply step during conversion
-        if (rawColor) {
-            decoder->image->alphaPremultiplied = AVIF_TRUE;
-        }
-        if (!avifJPEGWrite(outputFilename, decoder->image, jpegQuality, chromaUpsampling)) {
-            returnCode = 1;
-        }
-    } else if (outputFormat == AVIF_APP_FILE_FORMAT_PNG) {
-        if (!avifPNGWrite(outputFilename, decoder->image, requestedDepth, chromaUpsampling, pngCompressionLevel)) {
-            returnCode = 1;
-        }
+    const avifBool isSequence = decoder->imageCount > 1;
+    printf(" * %" PRIu64 " timescales per second, %2.2f seconds (%" PRIu64 " timescales), %d frame%s\n",
+           decoder->timescale,
+           decoder->duration,
+           decoder->durationInTimescales,
+           decoder->imageCount,
+           (decoder->imageCount == 1) ? "" : "s");
+    if (isSequence) {
+        printf(" * %s Frames: (%u expected frames)\n",
+               (decoder->progressiveState != AVIF_PROGRESSIVE_STATE_UNAVAILABLE) ? "Progressive Image" : "Image Sequence",
+               decoder->imageCount);
     } else {
-        fprintf(stderr, "Unsupported output file extension: %s\n", outputFilename);
-        returnCode = 1;
+        printf(" * Frame:\n");
     }
 
+    if (iccOverrideFilename) {
+        if (!avifReadEntireFile(iccOverrideFilename, &iccOverride)) {
+            fprintf(stderr, "ERROR: Unable to read ICC: %s\n", iccOverrideFilename);
+            avifRWDataFree(&iccOverride);
+            goto cleanup;
+        }
+    }
+
+    if (infoOnly && !frameIndexSpecified) {
+        frameIndex = DECODE_ALL_FRAMES; // Decode all frames by default in 'info only' mode.
+    }
+
+    const avifBool decodeAllFrames = frameIndex == DECODE_ALL_FRAMES;
+    int currIndex = decodeAllFrames ? 0 : frameIndex;
+    while (AVIF_TRUE) {
+        result = decodeAllFrames ? avifDecoderNextImage(decoder) : avifDecoderNthImage(decoder, frameIndex);
+        if (result != AVIF_RESULT_OK) {
+            break;
+        }
+
+        printf("   * Decoded frame [%d] [pts %2.2f (%" PRIu64 " timescales)] [duration %2.2f (%" PRIu64 " timescales)] [%ux%u]\n",
+               currIndex,
+               decoder->imageTiming.pts,
+               decoder->imageTiming.ptsInTimescales,
+               decoder->imageTiming.duration,
+               decoder->imageTiming.durationInTimescales,
+               decoder->image->width,
+               decoder->image->height);
+        if (infoOnly) {
+            ++currIndex;
+            if (decodeAllFrames) {
+                continue;
+            } else {
+                break;
+            }
+        }
+
+        if (decoder->image->transformFlags & AVIF_TRANSFORM_CLAP) {
+            avifCropRect cropRect;
+            if (!avifCropRectFromCleanApertureBox(&cropRect,
+                                                  &decoder->image->clap,
+                                                  decoder->image->width,
+                                                  decoder->image->height,
+                                                  &decoder->diag)) {
+                // Should happen only if AVIF_STRICT_CLAP_VALID is disabled.
+                fprintf(stderr, "Warning: Invalid Clean Aperture values\n");
+            }
+        }
+
+        if (ignoreICC && (decoder->image->icc.size > 0)) {
+            printf("[--ignore-icc] Discarding ICC profile.\n");
+            // This cannot fail.
+            result = avifImageSetProfileICC(decoder->image, NULL, 0);
+            assert(result == AVIF_RESULT_OK);
+        }
+
+        if (iccOverrideFilename) {
+            printf("[--icc] Setting ICC profile: %s\n", iccOverrideFilename);
+            result = avifImageSetProfileICC(decoder->image, iccOverride.data, iccOverride.size);
+            if (result != AVIF_RESULT_OK) {
+                fprintf(stderr, "ERROR: Failed to set ICC: %s\n", avifResultToString(result));
+                goto cleanup;
+            }
+        }
+
+        if (decodeAllFrames) {
+            // Create filename for individual frames, in the form path/to/output-0000000000.ext
+            char * lastDot = strrchr(outputFilename, '.');
+            const size_t dotPos = (lastDot != NULL) ? (size_t)(lastDot - outputFilename) : strlen(outputFilename);
+            const char * extension = (lastDot != NULL) ? lastDot + 1 : "";
+            const int maxFilenameWithoutExtensionLength = 1000;
+            const int maxExtensionLength = 10;
+            char frameFilename[1024];
+            int res = snprintf(frameFilename,
+                               sizeof(frameFilename),
+                               "%.*s-%010d.%.*s",
+                               ((int)dotPos > maxFilenameWithoutExtensionLength ? maxFilenameWithoutExtensionLength : (int)dotPos),
+                               outputFilename,
+                               currIndex,
+                               maxExtensionLength,
+                               extension);
+            if (res < 0) {
+                fprintf(stderr, "ERROR: Unable to generate output filename\n");
+                goto cleanup;
+            }
+            if (!avifWriteToFile(outputFormat, frameFilename, decoder->image, rawColor, jpegQuality, pngCompressionLevel, requestedDepth, chromaUpsampling)) {
+                goto cleanup;
+            }
+        } else {
+            if (!avifWriteToFile(outputFormat, outputFilename, decoder->image, rawColor, jpegQuality, pngCompressionLevel, requestedDepth, chromaUpsampling)) {
+                goto cleanup;
+            }
+            if (isSequence && !frameIndexSpecified) {
+                fprintf(stderr,
+                        "INFO: Decoded the first frame of an image sequence with %d frames. To output all frames, use --index all. To silence this message, use --index 0.\n",
+                        decoder->imageCount);
+            }
+            break;
+        }
+        ++currIndex;
+    }
+
+    if (result == AVIF_RESULT_NO_IMAGES_REMAINING) {
+        if (decodeAllFrames) {
+            result = AVIF_RESULT_OK;
+        } else {
+            fprintf(stderr,
+                    "ERROR: Frame at index %d requested but the file does not contain enough frames (signalled frame count: %d)\n",
+                    frameIndex,
+                    decoder->imageCount);
+            goto cleanup;
+        }
+    }
+    if (result != AVIF_RESULT_OK) {
+        fprintf(stderr, "ERROR: Failed to decode %s: %s\n", isSequence ? "frame" : "image", avifResultToString(result));
+        goto cleanup;
+    }
+
+    returnCode = 0;
+
 cleanup:
-    if (returnCode != 0) {
-        avifDumpDiagnostics(&decoder->diag);
+    if (decoder != NULL) {
+        if (returnCode != 0) {
+            avifDumpDiagnostics(&decoder->diag);
+        }
+        avifDecoderDestroy(decoder);
     }
-    avifDecoderDestroy(decoder);
+    avifRWDataFree(&iccOverride);
     return returnCode;
 }
