@@ -173,7 +173,7 @@ static avifBool avifExtractExifAndXMP(png_structp png, png_infop info, avifBool
             }
             avifRemoveHeader(&xmpApp1Header, &avif->xmp); // Ignore the return value because the header is optional.
             *ignoreXMP = AVIF_TRUE;                       // Ignore any other XMP chunk.
-        } else if (!strcmp(text->key, "Raw profile type APP1")) {
+        } else if (!strcmp(text->key, "Raw profile type APP1") || !strcmp(text->key, "Raw profile type app1")) { // ImageMagick uses lowercase app1.
             // This can be either Exif, XMP or something else.
             avifRWData metadata = { NULL, 0 };
             if (!avifCopyRawProfile(text->text, textLength, &metadata)) {
@@ -298,9 +298,7 @@ avifBool avifPNGRead(const char * inputFilename,
         png_set_tRNS_to_alpha(png);
     }
 
-    if ((rawColorType == PNG_COLOR_TYPE_GRAY) || (rawColorType == PNG_COLOR_TYPE_GRAY_ALPHA)) {
-        png_set_gray_to_rgb(png);
-    }
+    const avifBool rawColorTypeIsGray = (rawColorType == PNG_COLOR_TYPE_GRAY) || (rawColorType == PNG_COLOR_TYPE_GRAY_ALPHA);
 
     int imgBitDepth = 8;
     if (rawBitDepth == 16) {
@@ -322,7 +320,7 @@ avifBool avifPNGRead(const char * inputFilename,
         goto cleanup;
     }
     if (avif->yuvFormat == AVIF_PIXEL_FORMAT_NONE) {
-        if ((rawColorType == PNG_COLOR_TYPE_GRAY) || (rawColorType == PNG_COLOR_TYPE_GRAY_ALPHA)) {
+        if (rawColorTypeIsGray) {
             avif->yuvFormat = AVIF_PIXEL_FORMAT_YUV400;
         } else if (avif->matrixCoefficients == AVIF_MATRIX_COEFFICIENTS_IDENTITY ||
                    avif->matrixCoefficients == AVIF_MATRIX_COEFFICIENTS_YCGCO_RE) {
@@ -365,6 +363,18 @@ avifBool avifPNGRead(const char * inputFilename,
         // When the sRGB / iCCP chunk is present, applications that recognize it and are capable of color management
         // must ignore the gAMA and cHRM chunks and use the sRGB / iCCP chunk instead.
         if (png_get_iCCP(png, info, &iccpProfileName, &iccpCompression, &iccpData, &iccpDataLen) == PNG_INFO_iCCP) {
+            if (!rawColorTypeIsGray && avif->yuvFormat == AVIF_PIXEL_FORMAT_YUV400) {
+                fprintf(stderr,
+                        "The image contains a color ICC profile which is incompatible with the requested output "
+                        "format YUV400 (grayscale). Pass --ignore-icc to discard the ICC profile.\n");
+                goto cleanup;
+            }
+            if (rawColorTypeIsGray && avif->yuvFormat != AVIF_PIXEL_FORMAT_YUV400) {
+                fprintf(stderr,
+                        "The image contains a gray ICC profile which is incompatible with the requested output "
+                        "format YUV (color). Pass --ignore-icc to discard the ICC profile.\n");
+                goto cleanup;
+            }
             if (avifImageSetProfileICC(avif, iccpData, iccpDataLen) != AVIF_RESULT_OK) {
                 fprintf(stderr, "Setting ICC profile failed: out of memory.\n");
                 goto cleanup;
@@ -444,8 +454,8 @@ avifBool avifPNGRead(const char * inputFilename,
     }
 
     const int numChannels = png_get_channels(png, info);
-    if ((numChannels != 3) && (numChannels != 4)) {
-        fprintf(stderr, "png_get_channels() should return 3 or 4 but returns %d.\n", numChannels);
+    if (numChannels < 1 || numChannels > 4) {
+        fprintf(stderr, "png_get_channels() should return 1, 2, 3 or 4 but returns %d.\n", numChannels);
         goto cleanup;
     }
     if (avif->width > imageSizeLimit / avif->height) {
@@ -456,7 +466,11 @@ avifBool avifPNGRead(const char * inputFilename,
     avifRGBImageSetDefaults(&rgb, avif);
     rgb.chromaDownsampling = chromaDownsampling;
     rgb.depth = imgBitDepth;
-    if (numChannels == 3) {
+    if (numChannels == 1) {
+        rgb.format = AVIF_RGB_FORMAT_GRAY;
+    } else if (numChannels == 2) {
+        rgb.format = AVIF_RGB_FORMAT_GRAYA;
+    } else if (numChannels == 3) {
         rgb.format = AVIF_RGB_FORMAT_RGB;
     }
     if (avifRGBImageAllocatePixels(&rgb) != AVIF_RESULT_OK) {
@@ -558,12 +572,20 @@ avifBool avifPNGWrite(const char * outputFilename, const avifImage * avif, uint3
         colorType = PNG_COLOR_TYPE_GRAY;
     } else {
         avifRGBImageSetDefaults(&rgb, avif);
-        rgb.chromaUpsampling = chromaUpsampling;
         rgb.depth = rgbDepth;
-        colorType = PNG_COLOR_TYPE_RGBA;
-        if (avifImageIsOpaque(avif)) {
-            colorType = PNG_COLOR_TYPE_RGB;
-            rgb.format = AVIF_RGB_FORMAT_RGB;
+        if (avif->yuvFormat == AVIF_PIXEL_FORMAT_YUV400 && avif->alphaPlane) {
+            colorType = PNG_COLOR_TYPE_GRAY_ALPHA;
+            rgb.format = AVIF_RGB_FORMAT_GRAYA;
+        } else if (avif->yuvFormat == AVIF_PIXEL_FORMAT_YUV400 && !avif->alphaPlane) {
+            colorType = PNG_COLOR_TYPE_GRAY;
+            rgb.format = AVIF_RGB_FORMAT_GRAY;
+        } else {
+            rgb.chromaUpsampling = chromaUpsampling;
+            colorType = PNG_COLOR_TYPE_RGBA;
+            if (avifImageIsOpaque(avif)) {
+                colorType = PNG_COLOR_TYPE_RGB;
+                rgb.format = AVIF_RGB_FORMAT_RGB;
+            }
         }
         if (avifRGBImageAllocatePixels(&rgb) != AVIF_RESULT_OK) {
             fprintf(stderr, "Conversion to RGB failed: %s (out of memory)\n", outputFilename);
