@@ -217,7 +217,7 @@ static avifBool xyToXYZ(const float xy[2], double XYZ[3])
         return AVIF_FALSE;
     }
 
-    const double factor = 1 / xy[1];
+    const double factor = 1.0 / xy[1];
     XYZ[0] = xy[0] * factor;
     XYZ[1] = 1;
     XYZ[2] = (1 - xy[0] - xy[1]) * factor;
@@ -381,7 +381,7 @@ avifBool avifGenerateRGBICC(avifRWData * icc, float gamma, const float primaries
         return AVIF_FALSE;
     }
 
-    double rgbPrimaries[3][3] = {
+    const double rgbPrimaries[3][3] = {
         { primaries[0], primaries[2], primaries[4] },
         { primaries[1], primaries[3], primaries[5] },
         { 1.0 - primaries[0] - primaries[1], 1.0 - primaries[2] - primaries[3], 1.0 - primaries[4] - primaries[5] }
@@ -420,7 +420,9 @@ avifBool avifGenerateRGBICC(avifRWData * icc, float gamma, const float primaries
     matMul(adaptation, bradford, tmp);
 
     double bradfordInv[3][3];
-    matInv(bradford, bradfordInv);
+    if (!matInv(bradford, bradfordInv)) {
+        return AVIF_FALSE;
+    }
     matMul(bradfordInv, tmp, adaptation);
 
     double rgbXYZD50[3][3];
