@@ -7,10 +7,15 @@
 
 #include <assert.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "avif/avif.h"
+#include "avifexif.h"
+#include "avifutil.h"
+
 #define Y4M_MAX_LINE_SIZE 2048 // Arbitrary limit. Y4M headers should be much smaller than this
 
 struct y4mFrameIterator
@@ -67,11 +72,6 @@ static avifBool y4mColorSpaceParse(const char * formatString, struct y4mFrameIte
         frame->depth = 10;
         return AVIF_TRUE;
     }
-    if (!strcmp(formatString, "C422p10")) {
-        frame->format = AVIF_PIXEL_FORMAT_YUV422;
-        frame->depth = 10;
-        return AVIF_TRUE;
-    }
     if (!strcmp(formatString, "C444p12")) {
         frame->format = AVIF_PIXEL_FORMAT_YUV444;
         frame->depth = 12;
@@ -87,11 +87,6 @@ static avifBool y4mColorSpaceParse(const char * formatString, struct y4mFrameIte
         frame->depth = 12;
         return AVIF_TRUE;
     }
-    if (!strcmp(formatString, "C422p12")) {
-        frame->format = AVIF_PIXEL_FORMAT_YUV422;
-        frame->depth = 12;
-        return AVIF_TRUE;
-    }
     if (!strcmp(formatString, "C444")) {
         frame->format = AVIF_PIXEL_FORMAT_YUV444;
         frame->depth = 8;
@@ -132,6 +127,21 @@ static avifBool y4mColorSpaceParse(const char * formatString, struct y4mFrameIte
     return AVIF_FALSE;
 }
 
+// Returns an unsigned integer value parsed from [start:end[.
+// Returns -1 in case of failure.
+static int y4mReadUnsignedInt(const char * start, const char * end)
+{
+    const char * p = start;
+    int64_t value = 0;
+    while (p < end && *p >= '0' && *p <= '9') {
+        value = value * 10 + (*(p++) - '0');
+        if (value > INT_MAX) {
+            return -1;
+        }
+    }
+    return (p == start) ? -1 : (int)value;
+}
+
 // Note: this modifies framerateString
 static avifBool y4mFramerateParse(char * framerateString, avifAppSourceTiming * sourceTiming)
 {
@@ -243,7 +253,11 @@ static avifBool y4mClampSamples(avifImage * avif)
             goto cleanup; \
     } while (0)
 
-avifBool y4mRead(const char * inputFilename, avifImage * avif, avifAppSourceTiming * sourceTiming, struct y4mFrameIterator ** iter)
+avifBool y4mRead(const char * inputFilename,
+                 uint32_t imageSizeLimit,
+                 avifImage * avif,
+                 avifAppSourceTiming * sourceTiming,
+                 struct y4mFrameIterator ** iter)
 {
     avifBool result = AVIF_FALSE;
 
@@ -307,10 +321,10 @@ avifBool y4mRead(const char * inputFilename, avifImage * avif, avifAppSourceTimi
         while (p != end) {
             switch (*p) {
                 case 'W': // width
-                    frame.width = atoi((const char *)p + 1);
+                    frame.width = y4mReadUnsignedInt((const char *)p + 1, (const char *)end);
                     break;
                 case 'H': // height
-                    frame.height = atoi((const char *)p + 1);
+                    frame.height = y4mReadUnsignedInt((const char *)p + 1, (const char *)end);
                     break;
                 case 'C': // color space
                     if (!getHeaderString(p, end, tmpBuffer, 31)) {
@@ -381,6 +395,10 @@ avifBool y4mRead(const char * inputFilename, avifImage * avif, avifAppSourceTimi
         fprintf(stderr, "Failed to parse y4m header (not enough information): %s\n", frame.displayFilename);
         goto cleanup;
     }
+    if ((uint32_t)frame.width > imageSizeLimit / (uint32_t)frame.height) {
+        fprintf(stderr, "Too big y4m dimensions (%d x %d > %u px): %s\n", frame.width, frame.height, imageSizeLimit, frame.displayFilename);
+        goto cleanup;
+    }
 
     if (sourceTiming) {
         *sourceTiming = frame.sourceTiming;
@@ -438,7 +456,12 @@ cleanup:
             if (!feof(frame.inputFile)) {
                 // Remember y4m state for next time
                 *iter = malloc(sizeof(struct y4mFrameIterator));
-                **iter = frame;
+                if (*iter == NULL) {
+                    fprintf(stderr, "Inter-frame state memory allocation failure\n");
+                    result = AVIF_FALSE;
+                } else {
+                    **iter = frame;
+                }
             }
         }
     }
@@ -460,6 +483,27 @@ avifBool y4mWrite(const char * outputFilename, const avifImage * avif)
         fprintf(stderr, "WARNING: writing alpha is currently only supported in 8bpc YUV444, ignoring alpha channel: %s\n", outputFilename);
     }
 
+    if (avif->transformFlags & AVIF_TRANSFORM_CLAP) {
+        avifCropRect cropRect;
+        avifDiagnostics diag;
+        if (avifCropRectFromCleanApertureBox(&cropRect, &avif->clap, avif->width, avif->height, &diag) &&
+            (cropRect.x != 0 || cropRect.y != 0 || cropRect.width != avif->width || cropRect.height != avif->height)) {
+            // TODO: https://github.com/AOMediaCodec/libavif/issues/2427 - Implement.
+            fprintf(stderr,
+                    "Warning: Clean Aperture values were ignored, the output image was NOT cropped to rectangle {%u,%u,%u,%u}\n",
+                    cropRect.x,
+                    cropRect.y,
+                    cropRect.width,
+                    cropRect.height);
+        }
+    }
+    if (avifImageGetExifOrientationFromIrotImir(avif) != 1) {
+        // TODO: https://github.com/AOMediaCodec/libavif/issues/2427 - Rotate the samples.
+        fprintf(stderr,
+                "Warning: Orientation %u was ignored, the output image was NOT rotated or mirrored\n",
+                avifImageGetExifOrientationFromIrotImir(avif));
+    }
+
     switch (avif->depth) {
         case 8:
             switch (avif->yuvFormat) {
