@@ -16,9 +16,7 @@ if [ $# -ne 1 ]; then
   exit 1
 fi
 git clone -b 1.5.1 --depth 1 https://code.videolan.org/videolan/dav1d.git
-cd dav1d
-mkdir build
-cd build
+mkdir dav1d/build
 
 # This only works on linux and mac.
 if [ "$(uname)" == "Darwin" ]; then
@@ -32,13 +30,8 @@ ABI_LIST=("armeabi-v7a" "arm64-v8a" "x86" "x86_64")
 ARCH_LIST=("arm" "aarch64" "x86" "x86_64")
 for i in "${!ABI_LIST[@]}"; do
   abi="${ABI_LIST[i]}"
-  mkdir "${abi}"
-  cd "${abi}"
   PATH=$PATH:${android_bin} meson setup --default-library=static --buildtype release \
     --cross-file="../../package/crossfiles/${ARCH_LIST[i]}-android.meson" \
-    -Denable_tools=false -Denable_tests=false ../..
-  PATH=$PATH:${android_bin} ninja
-  cd ..
+    -Denable_tools=false -Denable_tests=false "dav1d/build/${abi}" dav1d
+  PATH=$PATH:${android_bin} meson compile -C "dav1d/build/${abi}"
 done
-
-cd ../..
