@@ -18,26 +18,35 @@
 # the test: source $(dirname "$0")/cmd_test_common.sh
 
 # Very verbose but useful for debugging.
-set -ex
+set -eux
 
-if [[ "$#" -ge 1 ]]; then
+# The CMake config must always be passed. It can be empty if there is
+# no multi-config generator.
+CONFIG="$1"
+if [[ "$#" -ge 2 ]]; then
   # eval so that the passed in directory can contain variables.
-  BINARY_DIR="$(eval echo "$1")"
+  BINARY_DIR="$(eval echo "$2")"
 else
   # Assume "tests" is the current directory.
   BINARY_DIR="$(pwd)/.."
 fi
-if [[ "$#" -ge 2 ]]; then
-  TESTDATA_DIR="$(eval echo "$2")"
+if [[ "$#" -ge 3 ]]; then
+  TESTDATA_DIR="$(eval echo "$3")"
 else
   TESTDATA_DIR="$(pwd)/data"
 fi
-if [[ "$#" -ge 3 ]]; then
-  TMP_DIR="$(eval echo "$3")"
+if [[ "$#" -ge 4 ]]; then
+  TMP_DIR="$(eval echo "$4")"
 else
   TMP_DIR="$(mktemp -d)"
 fi
 
-AVIFENC="${BINARY_DIR}/avifenc"
-AVIFDEC="${BINARY_DIR}/avifdec"
-ARE_IMAGES_EQUAL="${BINARY_DIR}/tests/are_images_equal"
+if [[ ! -z "$CONFIG" ]]; then
+  AVIFENC="${BINARY_DIR}/${CONFIG}/avifenc"
+  AVIFDEC="${BINARY_DIR}/${CONFIG}/avifdec"
+  ARE_IMAGES_EQUAL="${BINARY_DIR}/tests/${CONFIG}/are_images_equal"
+else
+  AVIFENC="${BINARY_DIR}/avifenc"
+  AVIFDEC="${BINARY_DIR}/avifdec"
+  ARE_IMAGES_EQUAL="${BINARY_DIR}/tests/are_images_equal"
+fi
