@@ -4,14 +4,18 @@
 #include "avifincrtest_helpers.h"
 
 #include <algorithm>
+#include <cmath>
+#include <cstdint>
 #include <cstring>
+#include <iostream>
+#include <memory>
 #include <vector>
 
 #include "avif/avif.h"
 #include "aviftest_helpers.h"
 #include "gtest/gtest.h"
 
-namespace libavif {
+namespace avif {
 namespace testutil {
 namespace {
 
@@ -60,28 +64,37 @@ void ComparePartialYuva(const avifImage& image1, const avifImage& image2,
       row2 += row2_bytes;
     }
   }
+
+  if (image1.gainMap != nullptr && image1.gainMap->image != nullptr &&
+      image2.gainMap != nullptr && image2.gainMap->image != nullptr) {
+    const uint32_t gain_map_row_count = (uint32_t)roundf(
+        (float)row_count / image1.height * image1.gainMap->image->height);
+    ComparePartialYuva(*image1.gainMap->image, *image2.gainMap->image,
+                       gain_map_row_count);
+  }
 }
 
 // Returns the expected number of decoded rows when available_byte_count out of
 // byte_count were given to the decoder, for an image of height rows, split into
 // cells of cell_height rows.
 uint32_t GetMinDecodedRowCount(uint32_t height, uint32_t cell_height,
-                               bool has_alpha, size_t available_byte_count,
-                               size_t byte_count,
+                               bool has_alpha, bool has_gain_map,
+                               size_t available_byte_count, size_t byte_count,
                                bool enable_fine_incremental_check) {
   // The whole image should be available when the full input is.
   if (available_byte_count >= byte_count) {
     return height;
   }
-  // All but one cell should be decoded if at most 10 bytes are missing.
+
+  // The tests below can be hard to tune for any kind of input, especially
+  // fuzzed grids, where tile ordering is unknown. Early exit in that case.
+  if (!enable_fine_incremental_check) return 0;
+
+  // There is no valid AV1 payload smaller than 10 bytes, so all but one cell
+  // should be decoded if at most 10 bytes are missing.
   if ((available_byte_count + 10) >= byte_count) {
     return height - cell_height;
   }
-
-  // The tests below can be hard to tune for any kind of input, especially
-  // fuzzed grids. Early exit in that case.
-  if (!enable_fine_incremental_check) return 0;
-
   // Subtract the header because decoding it does not output any pixel.
   // Most AVIF headers are below 500 bytes.
   if (available_byte_count <= 500) {
@@ -89,15 +102,17 @@ uint32_t GetMinDecodedRowCount(uint32_t height, uint32_t cell_height,
   }
   available_byte_count -= 500;
   byte_count -= 500;
-  // Alpha, if any, is assumed to be located before the other planes and to
-  // represent at most 50% of the payload.
-  if (has_alpha) {
-    if (available_byte_count <= (byte_count / 2)) {
-      return 0;
-    }
-    available_byte_count -= byte_count / 2;
-    byte_count -= byte_count / 2;
+  // Extra planes (alpha, gain map), if any, are assumed to be located before
+  // the color planes. It's assumed that each extra planes is at most
+  // total_size / (1 + num_extra_planes).
+  const int num_extra_planes = (has_alpha ? 1 : 0) + (has_gain_map ? 1 : 0);
+  const size_t max_size_of_extra_planes = static_cast<size_t>(
+      (byte_count / (num_extra_planes + 1)) * num_extra_planes);
+  if (available_byte_count <= max_size_of_extra_planes) {
+    return 0;
   }
+  available_byte_count -= max_size_of_extra_planes;
+  byte_count -= max_size_of_extra_planes;
   // Linearly map the input availability ratio to the decoded row ratio.
   const uint32_t min_decoded_cell_row_count = static_cast<uint32_t>(
       (height / cell_height) * available_byte_count / byte_count);
@@ -194,7 +209,7 @@ void EncodeAsGrid(const avifImage& image, uint32_t grid_cols,
     *cell_height = image.height / grid_rows;
   }
 
-  std::vector<testutil::AvifImagePtr> cell_images;
+  std::vector<ImagePtr> cell_images;
   cell_images.reserve(grid_cols * grid_rows);
   for (uint32_t row = 0, i_cell = 0; row < grid_rows; ++row) {
     for (uint32_t col = 0; col < grid_cols; ++col, ++i_cell) {
@@ -207,14 +222,14 @@ void EncodeAsGrid(const avifImage& image, uint32_t grid_cols,
       cell.height = ((cell.y + *cell_height) <= image.height)
                         ? *cell_height
                         : (image.height - cell.y);
-      cell_images.emplace_back(avifImageCreateEmpty(), avifImageDestroy);
+      cell_images.emplace_back(avifImageCreateEmpty());
       ASSERT_NE(cell_images.back(), nullptr);
       ASSERT_EQ(avifImageSetViewRect(cell_images.back().get(), &image, &cell),
                 AVIF_RESULT_OK);
     }
   }
 
-  testutil::AvifEncoderPtr encoder(avifEncoderCreate(), avifEncoderDestroy);
+  EncoderPtr encoder(avifEncoderCreate());
   ASSERT_NE(encoder, nullptr);
   encoder->speed = AVIF_SPEED_FASTEST;
   // Just here to match libavif API.
@@ -246,7 +261,7 @@ void EncodeRectAsIncremental(const avifImage& image, uint32_t width,
                              uint32_t height, bool create_alpha_if_none,
                              bool flat_cells, avifRWData* output,
                              uint32_t* cell_width, uint32_t* cell_height) {
-  AvifImagePtr sub_image(avifImageCreateEmpty(), avifImageDestroy);
+  ImagePtr sub_image(avifImageCreateEmpty());
   ASSERT_NE(sub_image, nullptr);
   ASSERT_LE(width, image.width);
   ASSERT_LE(height, image.height);
@@ -275,13 +290,16 @@ void EncodeRectAsIncremental(const avifImage& image, uint32_t width,
 
 //------------------------------------------------------------------------------
 
-void DecodeIncrementally(const avifRWData& encoded_avif, bool is_persistent,
-                         bool give_size_hint, bool use_nth_image_api,
-                         const avifImage& reference, uint32_t cell_height,
-                         bool enable_fine_incremental_check) {
+avifResult DecodeIncrementally(const avifRWData& encoded_avif,
+                               avifDecoder* decoder, bool is_persistent,
+                               bool give_size_hint, bool use_nth_image_api,
+                               const avifImage& reference, uint32_t cell_height,
+                               bool enable_fine_incremental_check,
+                               bool expect_whole_file_read,
+                               bool expect_parse_success_from_partial_file) {
   // AVIF cells are at least 64 pixels tall.
   if (cell_height != reference.height) {
-    ASSERT_GE(cell_height, 64u);
+    AVIF_CHECKERR(cell_height >= 64u, AVIF_RESULT_INVALID_ARGUMENT);
   }
 
   // Emulate a byte-by-byte stream.
@@ -292,73 +310,107 @@ void DecodeIncrementally(const avifRWData& encoded_avif, bool is_persistent,
       /*destroy=*/nullptr, PartialRead,
       /*write=*/nullptr,   give_size_hint ? encoded_avif.size : 0,
       is_persistent,       &data};
+  avifDecoderSetIO(decoder, &io);
+  // Reset the decoder's IO to nullptr before 'io' goes out of scope and becomes
+  // invalid.
+  auto cleanup_io_fn = [](avifDecoder* decoder) {
+    avifDecoderSetIO(decoder, nullptr);
+  };
+  std::unique_ptr<avifDecoder, decltype(cleanup_io_fn)> cleanup_io(
+      decoder, cleanup_io_fn);  // Call automatically at end of scope.
 
-  testutil::AvifDecoderPtr decoder(avifDecoderCreate(), avifDecoderDestroy);
-  ASSERT_NE(decoder, nullptr);
-  avifDecoderSetIO(decoder.get(), &io);
   decoder->allowIncremental = AVIF_TRUE;
   const size_t step = std::max<size_t>(1, data.full_size / 10000);
 
   // Parsing is not incremental.
-  avifResult parse_result = avifDecoderParse(decoder.get());
+  avifResult parse_result = avifDecoderParse(decoder);
   while (parse_result == AVIF_RESULT_WAITING_ON_IO) {
-    ASSERT_LT(data.available.size, data.full_size)
-        << "avifDecoderParse() returned WAITING_ON_IO instead of OK";
+    if (data.available.size >= data.full_size) {
+      std::cerr << "avifDecoderParse() returned WAITING_ON_IO instead of OK"
+                << std::endl;
+      return AVIF_RESULT_TRUNCATED_DATA;
+    }
     data.available.size = std::min(data.available.size + step, data.full_size);
-    parse_result = avifDecoderParse(decoder.get());
+    parse_result = avifDecoderParse(decoder);
   }
-  ASSERT_EQ(parse_result, AVIF_RESULT_OK);
+  if (data.available.size == data.full_size &&
+      expect_parse_success_from_partial_file) {
+    // Can happen if the data is in 'idat', or if some metadata is at the end of
+    // the file. But ideally this should be avoided.
+    printf(
+        "ERROR: had to provide the whole file for avifDecoderParse() to "
+        "succeed\n");
+    AVIF_CHECKERR(false, AVIF_RESULT_INVALID_ARGUMENT);
+  }
+  AVIF_CHECKRES(parse_result);
 
   // Decoding is incremental.
   uint32_t previously_decoded_row_count = 0;
   avifResult next_image_result = use_nth_image_api
-                                     ? avifDecoderNthImage(decoder.get(), 0)
-                                     : avifDecoderNextImage(decoder.get());
+                                     ? avifDecoderNthImage(decoder, 0)
+                                     : avifDecoderNextImage(decoder);
   while (next_image_result == AVIF_RESULT_WAITING_ON_IO) {
-    ASSERT_LT(data.available.size, data.full_size)
-        << (use_nth_image_api ? "avifDecoderNthImage(0)"
-                              : "avifDecoderNextImage()")
-        << " returned WAITING_ON_IO instead of OK";
-    const uint32_t decoded_row_count =
-        avifDecoderDecodedRowCount(decoder.get());
-    ASSERT_GE(decoded_row_count, previously_decoded_row_count);
+    if (data.available.size >= data.full_size) {
+      std::cerr << (use_nth_image_api ? "avifDecoderNthImage(0)"
+                                      : "avifDecoderNextImage()")
+                << " returned WAITING_ON_IO instead of OK";
+      AVIF_CHECKERR(false, AVIF_RESULT_INVALID_ARGUMENT);
+    }
+    const uint32_t decoded_row_count = avifDecoderDecodedRowCount(decoder);
+    if (decoded_row_count < previously_decoded_row_count) {
+      printf("ERROR: decoded row count decreased from %d to %d\n",
+             previously_decoded_row_count, decoded_row_count);
+      AVIF_CHECKERR(false, AVIF_RESULT_INVALID_ARGUMENT);
+    }
     const uint32_t min_decoded_row_count = GetMinDecodedRowCount(
         reference.height, cell_height, reference.alphaPlane != nullptr,
-        data.available.size, data.full_size, enable_fine_incremental_check);
-    ASSERT_GE(decoded_row_count, min_decoded_row_count);
+        reference.gainMap != nullptr, data.available.size, data.full_size,
+        enable_fine_incremental_check);
+    if (decoded_row_count < min_decoded_row_count) {
+      printf(
+          "ERROR: expected to have decoded at least %d rows with %zu available "
+          "bytes, but only %d were decoded\n",
+          min_decoded_row_count, data.available.size, decoded_row_count);
+      AVIF_CHECKERR(false, AVIF_RESULT_INVALID_ARGUMENT);
+    }
     ComparePartialYuva(reference, *decoder->image, decoded_row_count);
 
     previously_decoded_row_count = decoded_row_count;
     data.available.size = std::min(data.available.size + step, data.full_size);
-    next_image_result = use_nth_image_api
-                            ? avifDecoderNthImage(decoder.get(), 0)
-                            : avifDecoderNextImage(decoder.get());
+    next_image_result = use_nth_image_api ? avifDecoderNthImage(decoder, 0)
+                                          : avifDecoderNextImage(decoder);
   }
-  ASSERT_EQ(next_image_result, AVIF_RESULT_OK);
-  ASSERT_EQ(data.available.size, data.full_size);
-  ASSERT_EQ(avifDecoderDecodedRowCount(decoder.get()), decoder->image->height);
+  AVIF_CHECKRES(next_image_result);
+  if (expect_whole_file_read) {
+    AVIF_CHECKERR(data.available.size == data.full_size,
+                  AVIF_RESULT_INVALID_ARGUMENT);
+  }
+  AVIF_CHECKERR(avifDecoderDecodedRowCount(decoder) == decoder->image->height,
+                AVIF_RESULT_INVALID_ARGUMENT);
 
   ComparePartialYuva(reference, *decoder->image, reference.height);
+  return AVIF_RESULT_OK;
 }
 
-void DecodeNonIncrementallyAndIncrementally(
-    const avifRWData& encoded_avif, bool is_persistent, bool give_size_hint,
-    bool use_nth_image_api, uint32_t cell_height,
-    bool enable_fine_incremental_check) {
-  AvifImagePtr reference(avifImageCreateEmpty(), avifImageDestroy);
-  ASSERT_NE(reference, nullptr);
-  testutil::AvifDecoderPtr decoder(avifDecoderCreate(), avifDecoderDestroy);
-  ASSERT_NE(decoder, nullptr);
-  ASSERT_EQ(avifDecoderReadMemory(decoder.get(), reference.get(),
-                                  encoded_avif.data, encoded_avif.size),
-            AVIF_RESULT_OK);
+avifResult DecodeNonIncrementallyAndIncrementally(
+    const avifRWData& encoded_avif, avifDecoder* decoder, bool is_persistent,
+    bool give_size_hint, bool use_nth_image_api, uint32_t cell_height,
+    bool enable_fine_incremental_check, bool expect_whole_file_read,
+    bool expect_parse_success_from_partial_file) {
+  ImagePtr reference(avifImageCreateEmpty());
+  if (reference == nullptr) return AVIF_RESULT_INVALID_ARGUMENT;
+  decoder->allowIncremental = AVIF_FALSE;
+  AVIF_CHECKRES(avifDecoderReadMemory(decoder, reference.get(),
+                                      encoded_avif.data, encoded_avif.size));
 
-  DecodeIncrementally(encoded_avif, is_persistent, give_size_hint,
-                      use_nth_image_api, *reference, cell_height,
-                      enable_fine_incremental_check);
+  const avifResult result = DecodeIncrementally(
+      encoded_avif, decoder, is_persistent, give_size_hint, use_nth_image_api,
+      *reference, cell_height, enable_fine_incremental_check,
+      expect_whole_file_read, expect_parse_success_from_partial_file);
+  return result;
 }
 
 //------------------------------------------------------------------------------
 
 }  // namespace testutil
-}  // namespace libavif
+}  // namespace avif
