@@ -49,8 +49,8 @@ void ModifyImageChannel(avifRGBImage* image, uint32_t channel_offset,
 // Accumulates stats about the differences between the images a and b.
 template <typename PixelType>
 void GetDiffSumAndSqDiffSum(const avifRGBImage& a, const avifRGBImage& b,
-                            int64_t* diff_sum, int64_t* abs_diff_sum,
-                            int64_t* sq_diff_sum, int64_t* max_abs_diff) {
+                            int64_t* abs_diff_sum, int64_t* sq_diff_sum,
+                            int64_t* max_abs_diff) {
   const uint32_t channel_count = avifRGBFormatChannelCount(a.format);
   for (uint32_t y = 0; y < a.height; ++y) {
     const PixelType* row_a =
@@ -60,7 +60,6 @@ void GetDiffSumAndSqDiffSum(const avifRGBImage& a, const avifRGBImage& b,
     for (uint32_t x = 0; x < a.width * channel_count; ++x) {
       const int64_t diff =
           static_cast<int64_t>(row_b[x]) - static_cast<int64_t>(row_a[x]);
-      *diff_sum += diff;
       *abs_diff_sum += std::abs(diff);
       *sq_diff_sum += diff * diff;
       *max_abs_diff = std::max(*max_abs_diff, std::abs(diff));
@@ -69,12 +68,12 @@ void GetDiffSumAndSqDiffSum(const avifRGBImage& a, const avifRGBImage& b,
 }
 
 void GetDiffSumAndSqDiffSum(const avifRGBImage& a, const avifRGBImage& b,
-                            int64_t* diff_sum, int64_t* abs_diff_sum,
-                            int64_t* sq_diff_sum, int64_t* max_abs_diff) {
-  (a.depth <= 8) ? GetDiffSumAndSqDiffSum<uint8_t>(a, b, diff_sum, abs_diff_sum,
+                            int64_t* abs_diff_sum, int64_t* sq_diff_sum,
+                            int64_t* max_abs_diff) {
+  (a.depth <= 8) ? GetDiffSumAndSqDiffSum<uint8_t>(a, b, abs_diff_sum,
                                                    sq_diff_sum, max_abs_diff)
-                 : GetDiffSumAndSqDiffSum<uint16_t>(
-                       a, b, diff_sum, abs_diff_sum, sq_diff_sum, max_abs_diff);
+                 : GetDiffSumAndSqDiffSum<uint16_t>(a, b, abs_diff_sum,
+                                                    sq_diff_sum, max_abs_diff);
 }
 
 // Returns the Peak Signal-to-Noise Ratio from accumulated stats.
@@ -119,7 +118,7 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
                        avifMatrixCoefficients matrix_coefficients,
                        avifChromaDownsampling chroma_downsampling,
                        bool add_noise, uint32_t rgb_step,
-                       double max_abs_average_diff, double min_psnr, bool log) {
+                       double max_average_abs_diff, double min_psnr, bool log) {
   // Deduced constants.
   const bool is_monochrome =
       (yuv_format == AVIF_PIXEL_FORMAT_YUV400);  // If so, only test grey input.
@@ -146,7 +145,7 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
   }
 
   // Estimate the loss from converting RGB values to YUV and back.
-  int64_t diff_sum = 0, abs_diff_sum = 0, sq_diff_sum = 0, max_abs_diff = 0;
+  int64_t abs_diff_sum = 0, sq_diff_sum = 0, max_abs_diff = 0;
   int64_t num_diffs = 0;
   const uint32_t max_value = rgb_max - (add_noise ? 15 : 0);
   for (uint32_t r = 0; r < max_value + rgb_step; r += rgb_step) {
@@ -168,8 +167,8 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
 
       ASSERT_EQ(avifImageRGBToYUV(yuv.get(), &src_rgb), AVIF_RESULT_OK);
       ASSERT_EQ(avifImageYUVToRGB(yuv.get(), &dst_rgb), AVIF_RESULT_OK);
-      GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &diff_sum, &abs_diff_sum,
-                             &sq_diff_sum, &max_abs_diff);
+      GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &abs_diff_sum, &sq_diff_sum,
+                             &max_abs_diff);
       num_diffs += src_rgb.width * src_rgb.height * 3;  // Alpha is lossless.
     } else {
       for (uint32_t g = 0; g < max_value + rgb_step; g += rgb_step) {
@@ -193,8 +192,8 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
           }
           ASSERT_EQ(result, AVIF_RESULT_OK);
           ASSERT_EQ(avifImageYUVToRGB(yuv.get(), &dst_rgb), AVIF_RESULT_OK);
-          GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &diff_sum, &abs_diff_sum,
-                                 &sq_diff_sum, &max_abs_diff);
+          GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &abs_diff_sum, &sq_diff_sum,
+                                 &max_abs_diff);
           num_diffs +=
               src_rgb.width * src_rgb.height * 3;  // Alpha is lossless.
         }
@@ -206,13 +205,11 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
   // Note: The thresholds defined in this test are calibrated for libyuv fast
   //       paths. See reformat_libyuv.c. Slower non-libyuv conversions in
   //       libavif have a higher precision (using floating point operations).
-  const double average_diff =
-      static_cast<double>(diff_sum) / static_cast<double>(num_diffs);
   const double average_abs_diff =
       static_cast<double>(abs_diff_sum) / static_cast<double>(num_diffs);
   const double psnr = GetPsnr(static_cast<double>(sq_diff_sum),
                               static_cast<double>(num_diffs), rgb_max);
-  EXPECT_LE(std::abs(average_diff), max_abs_average_diff);
+  EXPECT_LE(average_abs_diff, max_average_abs_diff);
   EXPECT_GE(psnr, min_psnr);
 
   if (log) {
@@ -223,9 +220,9 @@ void ConvertWholeRange(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
               << kAvifRgbFormatToString[rgb_format] << ", "
               << avifPixelFormatToString(yuv_format) << ", "
               << (yuv_range ? "full" : "lmtd") << ", MC " << matrix_coefficients
-              << ", " << (add_noise ? "noisy" : "plain") << ", avg "
-              << average_diff << ", abs avg " << average_abs_diff << ", max "
-              << max_abs_diff << ", PSNR " << psnr << "dB" << std::endl;
+              << ", " << (add_noise ? "noisy" : "plain") << ", abs avg "
+              << average_abs_diff << ", max " << max_abs_diff << ", PSNR "
+              << psnr << "dB" << std::endl;
   }
 }
 
@@ -242,7 +239,7 @@ void ConvertWholeBuffer(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
   const uint32_t rgb_max = (1 << rgb_depth) - 1;
 
   // Estimate the loss from converting RGB values to YUV and back.
-  int64_t diff_sum = 0, abs_diff_sum = 0, sq_diff_sum = 0, max_abs_diff = 0;
+  int64_t abs_diff_sum = 0, sq_diff_sum = 0, max_abs_diff = 0;
   int64_t num_diffs = 0;
   for (int width : {1, 2, 127}) {
     for (int height : {1, 2, 251}) {
@@ -279,14 +276,11 @@ void ConvertWholeBuffer(int rgb_depth, int yuv_depth, avifRGBFormat rgb_format,
       }
       ASSERT_EQ(result, AVIF_RESULT_OK);
       ASSERT_EQ(avifImageYUVToRGB(yuv.get(), &dst_rgb), AVIF_RESULT_OK);
-      GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &diff_sum, &abs_diff_sum,
-                             &sq_diff_sum, &max_abs_diff);
+      GetDiffSumAndSqDiffSum(src_rgb, dst_rgb, &abs_diff_sum, &sq_diff_sum,
+                             &max_abs_diff);
       num_diffs += src_rgb.width * src_rgb.height * 3;
     }
   }
-  // max_abs_average_diff is not tested here because it is not meaningful for
-  // only 3*3 conversions as it takes the maximum difference per conversion.
-  // PSNR is averaged on all pixels so it can be tested here.
   EXPECT_GE(GetPsnr(static_cast<double>(sq_diff_sum),
                     static_cast<double>(num_diffs), rgb_max),
             min_psnr);
@@ -335,7 +329,7 @@ TEST(RGBToYUVTest, ExhaustiveSettings) {
                     // Just try min and max values.
                     /*rgb_step=*/(1u << rgb_depth) - 1u,
                     // Barely check the results, this is mostly for coverage.
-                    /*max_abs_average_diff=*/(1u << rgb_depth) - 1u,
+                    /*max_average_abs_diff=*/(1u << rgb_depth) - 1u,
                     /*min_psnr=*/5.0,
                     // Avoid spam.
                     /*log=*/false);
@@ -405,7 +399,7 @@ TEST(RGBToYUVTest, AllMatrixCoefficients) {
                   // Just try min and max values.
                   /*rgb_step=*/(1u << rgb_depth) - 1u,
                   // Barely check the results, this is mostly for coverage.
-                  /*max_abs_average_diff=*/(1u << rgb_depth) - 1u,
+                  /*max_average_abs_diff=*/(1u << rgb_depth) - 1u,
                   /*min_psnr=*/5.0,
                   // Avoid spam.
                   /*log=*/false);
@@ -417,6 +411,59 @@ TEST(RGBToYUVTest, AllMatrixCoefficients) {
   }
 }
 
+TEST(RGBToYUVTest, 8BitGrayToYUV420) {
+  // 2x2 8-bit image
+  static const uint8_t gray[4] = {4, 3, 2, 1};
+  ImagePtr image(avifImageCreate(2, 2, 8, AVIF_PIXEL_FORMAT_YUV420));
+  ASSERT_NE(image, nullptr);
+  avifRGBImage rgb;
+  avifRGBImageSetDefaults(&rgb, image.get());
+  rgb.format = AVIF_RGB_FORMAT_GRAY;
+  rgb.avoidLibYUV = AVIF_TRUE;
+  rgb.pixels = const_cast<uint8_t*>(gray);
+  rgb.rowBytes = 2 * sizeof(uint8_t);
+  ASSERT_EQ(avifImageRGBToYUV(image.get(), &rgb), AVIF_RESULT_OK);
+  const uint8_t* y_plane = image->yuvPlanes[AVIF_CHAN_Y];
+  const uint8_t* u_plane = image->yuvPlanes[AVIF_CHAN_U];
+  const uint8_t* v_plane = image->yuvPlanes[AVIF_CHAN_V];
+  EXPECT_EQ(y_plane[0], gray[0]);
+  EXPECT_EQ(y_plane[1], gray[1]);
+  EXPECT_EQ(y_plane[2], gray[2]);
+  EXPECT_EQ(y_plane[3], gray[3]);
+  EXPECT_EQ(u_plane[0], 128);
+  EXPECT_EQ(v_plane[0], 128);
+}
+
+TEST(RGBToYUVTest, HighBitDepthGrayToYUV420) {
+  // 2x2 10-bit, 12-bit, or 16-bit image
+  static const uint16_t gray[4] = {4, 3, 2, 1};
+  static const uint32_t depth[3] = {10, 12, 16};
+  static const uint16_t half[3] = {512, 2048, 32768};
+  for (int i = 0; i < 3; i++) {
+    ImagePtr image(avifImageCreate(2, 2, depth[i], AVIF_PIXEL_FORMAT_YUV420));
+    ASSERT_NE(image, nullptr);
+    avifRGBImage rgb;
+    avifRGBImageSetDefaults(&rgb, image.get());
+    rgb.format = AVIF_RGB_FORMAT_GRAY;
+    rgb.avoidLibYUV = AVIF_TRUE;
+    rgb.pixels = reinterpret_cast<uint8_t*>(const_cast<uint16_t*>(gray));
+    rgb.rowBytes = 2 * sizeof(uint16_t);
+    ASSERT_EQ(avifImageRGBToYUV(image.get(), &rgb), AVIF_RESULT_OK);
+    const uint16_t* y_plane =
+        reinterpret_cast<uint16_t*>(image->yuvPlanes[AVIF_CHAN_Y]);
+    const uint16_t* u_plane =
+        reinterpret_cast<uint16_t*>(image->yuvPlanes[AVIF_CHAN_U]);
+    const uint16_t* v_plane =
+        reinterpret_cast<uint16_t*>(image->yuvPlanes[AVIF_CHAN_V]);
+    EXPECT_EQ(y_plane[0], gray[0]);
+    EXPECT_EQ(y_plane[1], gray[1]);
+    EXPECT_EQ(y_plane[2], gray[2]);
+    EXPECT_EQ(y_plane[3], gray[3]);
+    EXPECT_EQ(u_plane[0], half[i]);
+    EXPECT_EQ(v_plane[0], half[i]);
+  }
+}
+
 //------------------------------------------------------------------------------
 // Selected configurations
 
@@ -425,7 +472,7 @@ class RGBToYUVTest
           /*rgb_depth=*/int, /*yuv_depth=*/int, avifRGBFormat, avifPixelFormat,
           avifRange, avifMatrixCoefficients, avifChromaDownsampling,
           /*add_noise=*/bool, /*rgb_step=*/uint32_t,
-          /*max_abs_average_diff=*/double, /*min_psnr=*/double>> {};
+          /*max_average_abs_diff=*/double, /*min_psnr=*/double>> {};
 
 TEST_P(RGBToYUVTest, ConvertWholeRange) {
   ConvertWholeRange(
@@ -443,7 +490,7 @@ TEST_P(RGBToYUVTest, ConvertWholeRange) {
       // similar with faster settings.
       /*rgb_step=*/std::get<8>(GetParam()),
       // Thresholds to pass.
-      /*max_abs_average_diff=*/std::get<9>(GetParam()),
+      /*max_average_abs_diff=*/std::get<9>(GetParam()),
       /*min_psnr=*/std::get<10>(GetParam()),
       // Useful to see surrounding results when there is a failure.
       /*log=*/true);
@@ -486,8 +533,7 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Values(true),
             /*rgb_step=*/Values(3),
-            /*max_abs_average_diff=*/Values(0.1),  // The color drift is almost
-                                                   // centered.
+            /*max_average_abs_diff=*/Values(2.88),
             /*min_psnr=*/Values(36.)  // Subsampling distortion is acceptable.
             ));
 
@@ -503,7 +549,7 @@ INSTANTIATE_TEST_SUITE_P(Identity8b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(true),
                                  /*rgb_step=*/Values(31),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(Identity10b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(10),
@@ -515,7 +561,7 @@ INSTANTIATE_TEST_SUITE_P(Identity10b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(true),
                                  /*rgb_step=*/Values(101),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(Identity12b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(12),
@@ -527,7 +573,7 @@ INSTANTIATE_TEST_SUITE_P(Identity12b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(true),
                                  /*rgb_step=*/Values(401),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(Identity16b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(16),
@@ -539,7 +585,7 @@ INSTANTIATE_TEST_SUITE_P(Identity16b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(true),
                                  /*rgb_step=*/Values(6421),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 
 // 4:4:4 and chroma subsampling have similar distortions on plain color inputs.
@@ -553,7 +599,7 @@ INSTANTIATE_TEST_SUITE_P(
         Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
         /*add_noise=*/Values(false),
         /*rgb_step=*/Values(17),
-        /*max_abs_average_diff=*/Values(0.02),  // The color drift is centered.
+        /*max_average_abs_diff=*/Values(0.84),
         /*min_psnr=*/Values(45.)  // RGB>YUV>RGB distortion is barely
                                   // noticeable.
         ));
@@ -570,7 +616,7 @@ INSTANTIATE_TEST_SUITE_P(MonochromeLossless8b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(false),
                                  /*rgb_step=*/Values(1),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(MonochromeLossless10b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(10),
@@ -582,7 +628,7 @@ INSTANTIATE_TEST_SUITE_P(MonochromeLossless10b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(false),
                                  /*rgb_step=*/Values(1),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(MonochromeLossless12b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(12),
@@ -594,7 +640,7 @@ INSTANTIATE_TEST_SUITE_P(MonochromeLossless12b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(false),
                                  /*rgb_step=*/Values(1),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 INSTANTIATE_TEST_SUITE_P(MonochromeLossless16b, RGBToYUVTest,
                          Combine(/*rgb_depth=*/Values(16),
@@ -606,7 +652,7 @@ INSTANTIATE_TEST_SUITE_P(MonochromeLossless16b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(false),
                                  /*rgb_step=*/Values(401),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 
 // Tests YCGCO_RE is lossless.
@@ -620,7 +666,7 @@ INSTANTIATE_TEST_SUITE_P(YCgCo_Re8b, RGBToYUVTest,
                                  Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
                                  /*add_noise=*/Values(true),
                                  /*rgb_step=*/Values(101),
-                                 /*max_abs_average_diff=*/Values(0.),
+                                 /*max_average_abs_diff=*/Values(0.),
                                  /*min_psnr=*/Values(99.)));
 
 // Coverage for reformat_libsharpyuv.c.
@@ -634,8 +680,8 @@ INSTANTIATE_TEST_SUITE_P(
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
         /*rgb_step=*/Values(17),
-        /*max_abs_average_diff=*/Values(1.2),  // Sharp YUV introduces some
-                                               // color shift.
+        /*max_average_abs_diff=*/Values(2.97),  // Sharp YUV introduces some
+                                                // color shift.
         /*min_psnr=*/Values(34.)  // SharpYuv distortion is acceptable.
         ));
 INSTANTIATE_TEST_SUITE_P(
@@ -649,8 +695,8 @@ INSTANTIATE_TEST_SUITE_P(
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
         /*rgb_step=*/Values(17),
-        /*max_abs_average_diff=*/Values(1.2),  // Sharp YUV introduces some
-                                               // color shift.
+        /*max_average_abs_diff=*/Values(2.94),  // Sharp YUV introduces some
+                                                // color shift.
         /*min_psnr=*/Values(34.)  // SharpYuv distortion is acceptable.
         ));
 INSTANTIATE_TEST_SUITE_P(
@@ -663,8 +709,8 @@ INSTANTIATE_TEST_SUITE_P(
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
         /*rgb_step=*/Values(17),
-        /*max_abs_average_diff=*/Values(1.2),  // Sharp YUV introduces some
-                                               // color shift.
+        /*max_average_abs_diff=*/Values(2.94),  // Sharp YUV introduces some
+                                                // color shift.
         /*min_psnr=*/Values(34.)  // SharpYuv distortion is acceptable.
         ));
 INSTANTIATE_TEST_SUITE_P(
@@ -676,9 +722,9 @@ INSTANTIATE_TEST_SUITE_P(
         Values(kMatrixCoefficientsBT601),
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
-        /*rgb_step=*/Values(211),              // High or it would be too slow.
-        /*max_abs_average_diff=*/Values(1.2),  // Sharp YUV introduces some
-                                               // color shift.
+        /*rgb_step=*/Values(211),               // High or it would be too slow.
+        /*max_average_abs_diff=*/Values(2.94),  // Sharp YUV introduces some
+                                                // color shift.
         /*min_psnr=*/Values(34.)  // SharpYuv distortion is acceptable.
         ));
 INSTANTIATE_TEST_SUITE_P(
@@ -690,24 +736,23 @@ INSTANTIATE_TEST_SUITE_P(
         Values(kMatrixCoefficientsBT601),
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
-        /*rgb_step=*/Values(840),              // High or it would be too slow.
-        /*max_abs_average_diff=*/Values(1.2),  // Sharp YUV introduces some
-                                               // color shift.
+        /*rgb_step=*/Values(840),               // High or it would be too slow.
+        /*max_average_abs_diff=*/Values(6.57),  // Sharp YUV introduces some
+                                                // color shift.
         /*min_psnr=*/Values(34.)  // SharpYuv distortion is acceptable.
         ));
 INSTANTIATE_TEST_SUITE_P(
     SharpYuv16Bit, RGBToYUVTest,
     Combine(
         /*rgb_depth=*/Values(16),
-        // TODO(yguyon): Why max_abs_average_diff>28 if RGB16 to YUV10 full rng?
         /*yuv_depth=*/Values(8, /*10,*/ 12), Values(AVIF_RGB_FORMAT_RGBA),
         Values(AVIF_PIXEL_FORMAT_YUV420), Values(AVIF_RANGE_FULL),
         Values(kMatrixCoefficientsBT601),
         Values(AVIF_CHROMA_DOWNSAMPLING_SHARP_YUV),
         /*add_noise=*/Values(true),
-        /*rgb_step=*/Values(4567),             // High or it would be too slow.
-        /*max_abs_average_diff=*/Values(2.5),  // Sharp YUV introduces some
-                                               // color shift.
+        /*rgb_step=*/Values(4567),  // High or it would be too slow.
+        /*max_average_abs_diff=*/Values(111.7),  // Sharp YUV introduces some
+                                                 // color shift.
         /*min_psnr=*/Values(49.)  // SharpYuv distortion is acceptable.
         ));
 
@@ -724,8 +769,8 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Bool(),
             /*rgb_step=*/Values(61),  // High or it would be too slow.
-            /*max_abs_average_diff=*/Values(1.),  // Not very accurate because
-                                                  // of high rgb_step.
+            /*max_average_abs_diff=*/Values(2.96),  // Not very accurate because
+                                                    // of high rgb_step.
             /*min_psnr=*/Values(36.)));
 INSTANTIATE_TEST_SUITE_P(
     All10b, RGBToYUVTest,
@@ -736,8 +781,8 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Bool(),
             /*rgb_step=*/Values(211),  // High or it would be too slow.
-            /*max_abs_average_diff=*/Values(0.2),  // Not very accurate because
-                                                   // of high rgb_step.
+            /*max_average_abs_diff=*/Values(2.83),  // Not very accurate because
+                                                    // of high rgb_step.
             /*min_psnr=*/Values(47.)));
 INSTANTIATE_TEST_SUITE_P(
     All12b, RGBToYUVTest,
@@ -748,8 +793,8 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Bool(),
             /*rgb_step=*/Values(809),  // High or it would be too slow.
-            /*max_abs_average_diff=*/Values(0.3),  // Not very accurate because
-                                                   // of high rgb_step.
+            /*max_average_abs_diff=*/Values(2.82),  // Not very accurate because
+                                                    // of high rgb_step.
             /*min_psnr=*/Values(52.)));
 INSTANTIATE_TEST_SUITE_P(
     All16b, RGBToYUVTest,
@@ -760,7 +805,7 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Bool(),
             /*rgb_step=*/Values(16001),  // High or it would be too slow.
-            /*max_abs_average_diff=*/Values(0.05),
+            /*max_average_abs_diff=*/Values(2.82),
             /*min_psnr=*/Values(80.)));
 
 // This was used to estimate the quality loss of libyuv for RGB-to-YUV.
@@ -776,7 +821,7 @@ INSTANTIATE_TEST_SUITE_P(
             Values(AVIF_CHROMA_DOWNSAMPLING_AUTOMATIC),
             /*add_noise=*/Bool(),
             /*rgb_step=*/Values(3),  // way faster and 99% similar to rgb_step=1
-            /*max_abs_average_diff=*/Values(10.),
+            /*max_average_abs_diff=*/Values(10.),
             /*min_psnr=*/Values(10.)));
 
 }  // namespace
