@@ -16,7 +16,7 @@
 #
 # tests for command lines
 
-source $(dirname "$0")/cmd_test_common.sh
+source $(dirname "$0")/cmd_test_common.sh || exit
 
 # Basic calls.
 "${AVIFENC}" --version
